"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendSignedOptOutTransaction = exports.createOptOutTransaction = void 0;
const ethers_1 = require("ethers");
const utils_1 = require("../utils");
const utils_2 = require("./utils");
const utils_3 = require("./utils");
const distributionAbi = [{
        "type": "function",
        "stateMutability": "nonpayable",
        "outputs": [],
        "name": "optOutOfAirdrop",
        "inputs": []
    }];
/**
 * @description Creates the opt out transaction and stores unsigned trx object in the file id that was passed
 * @param ctx - context
 * @param id - file id
 * @param nonce - nonce
 * @returns returns the file id
 */
function createOptOutTransaction(ctx, id, nonce) {
    return __awaiter(this, void 0, void 0, function* () {
        const distributionAddress = ctx.config.networkID == 14
            ? "0x9c7A4C83842B29bB4A082b0E689CB9474BD938d0"
            : "0xbd33bDFf04C357F7FC019E72D0504C24CF4Aa010";
        const txNonce = nonce !== null && nonce !== void 0 ? nonce : yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex);
        const distributionWeb3Contract = (0, utils_2.getWeb3Contract)(ctx.web3, distributionAddress, distributionAbi);
        const fnToEncode = distributionWeb3Contract.methods.optOutOfAirdrop();
        const rawTx = {
            nonce: txNonce,
            gasPrice: 200000000000,
            gasLimit: 4000000,
            to: distributionWeb3Contract.options.address,
            data: fnToEncode.encodeABI(),
            chainId: ctx.config.networkID
        };
        // serialized unsigned transaction
        const ethersTx = ethers_1.Transaction.from(rawTx);
        const hash = (0, utils_1.unPrefix0x)(ethersTx.unsignedHash);
        const unsignedTx = {
            rawTx: rawTx,
            message: hash,
            forDefiHash: Buffer.from(hash, 'hex').toString('base64')
        };
        // save tx data
        (0, utils_3.saveUnsignedWithdrawalTx)(unsignedTx, id);
        return id;
    });
}
exports.createOptOutTransaction = createOptOutTransaction;
/**
 * @description - sends the opt out transaction to the blockchain
 * @param ctx - context
 * @param id - id of the file containing the unsigned transaction
 * @returns - the transaction hash
 */
function sendSignedOptOutTransaction(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const waitFinalize3 = (0, utils_3.waitFinalize3Factory)(ctx.web3);
        // read unsigned tx data
        const unsignedTxJson = (0, utils_3.readUnsignedWithdrawalTx)(id);
        // read signed tx data
        const signedTxJson = (0, utils_3.readSignedWithdrawalTx)(id);
        // read signature
        const signature = signedTxJson.signature;
        // create raw signed tx
        const ethersTx = ethers_1.Transaction.from(unsignedTxJson.rawTx);
        ethersTx.signature = (0, utils_1.prefix0x)(signature);
        const serializedSigned = ethersTx.serialized;
        // send signed tx to the network
        const receipt = yield waitFinalize3(ctx.cAddressHex, () => ctx.web3.eth.sendSignedTransaction(serializedSigned));
        return receipt.transactionHash;
    });
}
exports.sendSignedOptOutTransaction = sendSignedOptOutTransaction;
