"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendSignedWithdrawalTransaction = exports.createWithdrawalTransaction = void 0;
const ethers_1 = require("ethers");
const utils_1 = require("../utils");
const utils_2 = require("./utils");
/**
 * @description Creates the withdrawal transaction and stores unsigned trx object in the file id that was passed
 * @param ctx - context
 * @param toAddress - the to address
 * @param amount - amount to be withdrawan
 * @param id - file id
 * @param nonce - nonce
 * @returns returns the file id
 */
function createWithdrawalTransaction(ctx, toAddress, amount, id, nonce) {
    return __awaiter(this, void 0, void 0, function* () {
        const txNonce = (nonce === undefined) ? yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex) : nonce;
        let amountWei = BigInt(amount) * BigInt(10 ** 9); // amount has already 9 zeros
        var rawTx = {
            nonce: txNonce,
            gasPrice: 200000000000,
            gasLimit: 21000,
            to: toAddress,
            value: amountWei.toString(),
            chainId: ctx.config.networkID
        };
        // serialized unsigned transaction
        const ethersTx = ethers_1.Transaction.from(rawTx);
        let hash = (0, utils_1.unPrefix0x)(ethersTx.unsignedHash);
        const unsignedTx = {
            rawTx: rawTx,
            message: hash,
            forDefiHash: Buffer.from(hash, 'hex').toString('base64')
        };
        // save tx data
        (0, utils_2.saveUnsignedWithdrawalTx)(unsignedTx, id);
        return id;
    });
}
exports.createWithdrawalTransaction = createWithdrawalTransaction;
/**
 * @description - sends the withdrawal transaction to the blockchain
 * @param ctx - context
 * @param id - id of the file containig the unsigned transaction
 * @returns - the transaction hash
 */
function sendSignedWithdrawalTransaction(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const waitFinalize3 = (0, utils_2.waitFinalize3Factory)(ctx.web3);
        // read unsigned tx data
        const unsignedTxJson = (0, utils_2.readUnsignedWithdrawalTx)(id);
        // read signed tx data
        const signedTxJson = (0, utils_2.readSignedWithdrawalTx)(id);
        // read signature
        const signature = signedTxJson.signature;
        // create raw signed tx
        const ethersTx = ethers_1.Transaction.from(unsignedTxJson.rawTx);
        ethersTx.signature = (0, utils_1.prefix0x)(signature);
        const serializedSigned = ethersTx.serialized;
        // send signed tx to the network
        let receipt = yield waitFinalize3(ctx.cAddressHex, () => ctx.web3.eth.sendSignedTransaction(serializedSigned));
        return receipt.transactionHash;
    });
}
exports.sendSignedWithdrawalTransaction = sendSignedWithdrawalTransaction;
