"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.prompts = void 0;
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = __importDefault(require("inquirer"));
const screen_1 = require("../constants/screen");
/**
 * Provides various prompts used in the CLI for user interaction.
 */
exports.prompts = {
    connectWallet: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'wallet',
                message: chalk_1.default.magenta("How do you want to connect your wallet?"),
                choices: [...Object.values(screen_1.walletConstants)],
                filter: (val) => {
                    const key = Object.keys(screen_1.walletConstants).find(key => screen_1.walletConstants[key] == val);
                    return key;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    derivationType: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'derivation',
                message: chalk_1.default.magenta("Choose derivation path..."),
                choices: [
                    ...Object.values(screen_1.derivationModeConstants)
                ],
                filter: (val) => {
                    const key = Object.keys(screen_1.derivationModeConstants).find(key => screen_1.derivationModeConstants[key] == val);
                    return key;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    pvtKeyPath: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [
            {
                type: 'input',
                name: 'pvtKeyPath',
                message: chalk_1.default.magenta("Enter Path to Private Key file") + " " +
                    chalk_1.default.yellow("(E.g. /home/wallet/pvtKeyFile)") +
                    chalk_1.default.magenta(":"),
            },
        ];
        return inquirer_1.default.prompt(questions);
    }),
    publicKey: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [
            {
                type: 'input',
                name: 'publicKey',
                message: chalk_1.default.magenta("Enter your secp256k1 curve public key") + " " +
                    chalk_1.default.yellow("(E.g. 0x02efe41c5d213089cb7a9e808505e9084bb9eb2bf3aa8050ea92a5ae9e20e5a692)") +
                    chalk_1.default.magenta(":"),
            },
        ];
        return inquirer_1.default.prompt(questions);
    }),
    amount: (amountPurpose = "") => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [
            {
                type: 'input',
                name: 'amount',
                message: chalk_1.default.magenta(`Enter amount ${amountPurpose}`) + " " +
                    chalk_1.default.magenta(`(in FLR):`)
            },
        ];
        return inquirer_1.default.prompt(questions);
    }),
    fees: (baseFees) => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                default: baseFees,
                name: 'fees',
                message: chalk_1.default.magenta("Enter fees (in FLR):")
            }];
        return inquirer_1.default.prompt(questions);
    }),
    nodeId: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'id',
                message: chalk_1.default.magenta("Enter NodeId") + " " +
                    chalk_1.default.yellow("(E.g. NodeID-FQKTLuZHEsjCxPeFTFgsojsucmdyNDsz1)") +
                    chalk_1.default.magenta(":")
            }];
        return inquirer_1.default.prompt(questions);
    }),
    unixTime: (timeType) => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'time',
                message: chalk_1.default.magenta(`Enter ${timeType} time`) + " " +
                    chalk_1.default.yellow("(E.g. 1693185095") +
                    chalk_1.default.magenta(":")
            }];
        return inquirer_1.default.prompt(questions);
    }),
    ctxFile: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'isContinue',
                message: chalk_1.default.magenta("Do you wish to continue with this?"),
                choices: [
                    "yes",
                    "no"
                ],
                filter: (val) => {
                    return val == "yes" ? true : false;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    selectNetwork: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'network',
                message: chalk_1.default.magenta("Which network do you want to connect to?"),
                choices: [
                    ...Object.values(screen_1.networkConstants)
                ],
                filter: (val) => {
                    const key = Object.keys(screen_1.networkConstants).find(key => screen_1.networkConstants[key] == val);
                    return key;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    selectTask: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'task',
                message: chalk_1.default.magenta("What do you want to do?"),
                choices: [
                    ...Object.keys(screen_1.taskConstants)
                ]
            }];
        return inquirer_1.default.prompt(questions);
    }),
    selectAddress: (choiceList) => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'address',
                message: chalk_1.default.magenta("Which address do you want to use?"),
                choices: [
                    ...choiceList
                ],
                filter: (val) => {
                    const address = val.split(" ")[1];
                    return address;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    delegationFee: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'fee',
                message: chalk_1.default.magenta("Enter delegation fee") + " " +
                    chalk_1.default.yellow("(E.g. 10)") +
                    chalk_1.default.magenta(":")
            }];
        return inquirer_1.default.prompt(questions);
    }),
    vaultId: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'id',
                message: chalk_1.default.magenta("Enter a ForDefi VauldId") + " " +
                    chalk_1.default.yellow("(E.g. 42989fc9-xxxx-xxxx-xxxx-xxxxxxxxxxxx)") +
                    chalk_1.default.magenta(":")
            }];
        return inquirer_1.default.prompt(questions);
    }),
    transactionId: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'id',
                message: chalk_1.default.magenta("Enter Transaction Id") + " " +
                    chalk_1.default.yellow("(E.g. abc-txn)") +
                    chalk_1.default.magenta(":"),
            }];
        return inquirer_1.default.prompt(questions);
    }),
    forDefiTxn: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'txn',
                message: chalk_1.default.magenta("Which transaction do you want to do?"),
                choices: [
                    "Export funds",
                    "Import funds"
                ],
            }];
        return inquirer_1.default.prompt(questions);
    }),
    forDefiContinue: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'isContinue',
                message: chalk_1.default.magenta("Choose an option"),
                choices: [
                    "Start new transaction",
                    "Continue existing transaction"
                ],
                filter: (val) => {
                    return val.includes("existing") ? true : false;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    isOwnerReceiver: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'isOwnerReceiver',
                message: chalk_1.default.magenta("Where do you want to receive your rewards?"),
                choices: [
                    "Send to my wallet",
                    "Receive with another wallet"
                ],
                filter: (val) => {
                    return val.includes("my") ? true : false;
                }
            }];
        return inquirer_1.default.prompt(questions);
    }),
    receiverAddress: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'address',
                message: chalk_1.default.magenta("Please enter the C-address where you want to receive your rewards:"),
            }];
        return inquirer_1.default.prompt(questions);
    }),
    withdrawAddress: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'input',
                name: 'address',
                message: chalk_1.default.magenta("Please enter the C-address where you want to withdraw your funds:"),
            }];
        return inquirer_1.default.prompt(questions);
    }),
    importTrxType: () => __awaiter(void 0, void 0, void 0, function* () {
        const questions = [{
                type: 'list',
                name: 'type',
                message: chalk_1.default.magenta("Please select the destination chain to which you want to import your funds?"),
                choices: [
                    "P",
                    "C"
                ],
            }];
        return inquirer_1.default.prompt(questions);
    }),
};
