"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTransportPath = exports.ledgerGetAccount = void 0;
const hw_app_avalanche_1 = __importDefault(require("@avalabs/hw-app-avalanche"));
const hw_transport_node_hid_1 = __importDefault(require("@ledgerhq/hw-transport-node-hid"));
let transport;
/**
 *
 * @param derivationPath - path to the accounts in ledger
 * @param hrp - network hrp
 * @returns returns the address and public key
 */
function ledgerGetAccount(derivationPath, hrp) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!transport) {
            transport = yield getTransportPath();
        }
        const avalanche = new hw_app_avalanche_1.default(transport);
        const pubkaddr = yield avalanche.getAddressAndPubKey(derivationPath, false, hrp);
        if (pubkaddr.errorMessage != 'No errors') {
            throw Error(`Can not get address from ledger: ${pubkaddr.errorMessage}, code ${pubkaddr.returnCode}`);
        }
        return {
            publicKey: pubkaddr.publicKey.toString('hex'),
            address: pubkaddr.address
        };
    });
}
exports.ledgerGetAccount = ledgerGetAccount;
/**
 * @description Opens a transport path to HID and returns an instance of class TransportNodeHid
 * @returns {TransportNodeHid}
 */
function getTransportPath() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!transport) {
            transport = yield hw_transport_node_hid_1.default.open(undefined);
        }
        return transport;
    });
}
exports.getTransportPath = getTransportPath;
