"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const chalk_1 = __importDefault(require("chalk"));
const clear_1 = __importDefault(require("clear"));
const figlet_1 = __importDefault(require("figlet"));
const cli_1 = require("./cli");
const cli_2 = require("./interactive/cli");
const package_json_1 = require("../package.json");
function cliInfo(isClear = false) {
    if (isClear) {
        (0, clear_1.default)();
    }
    console.log(chalk_1.default.white(figlet_1.default.textSync('Flare Stake CLI')));
    console.log(chalk_1.default.green(`Version: ${package_json_1.version}`));
}
const baseArgv = process.argv;
const command = baseArgv[2];
function getArgv() {
    if (command === 'exportCP' ||
        command === 'importCP' ||
        command === 'exportPC' ||
        command === 'importPC' ||
        command === 'stake' ||
        command === 'delegate') {
        return [...baseArgv.slice(0, 2), 'transaction', ...baseArgv.slice(2)];
    }
    else if (command === 'addresses' ||
        command === 'network' ||
        command === 'balance' ||
        command === 'validators') {
        return [...baseArgv.slice(0, 2), 'info', ...baseArgv.slice(2)];
    }
    else {
        return baseArgv;
    }
}
if (command == 'interactive' || command == "-i") {
    cliInfo(true);
    (0, cli_2.interactiveCli)(baseArgv).then(() => {
        console.log(`Finished execution`);
    }).catch((error) => {
        console.log(chalk_1.default.red("E"), error);
    });
}
else {
    cliInfo();
    const program = new commander_1.Command("Flare Stake Tool");
    (0, cli_1.cli)(program).then(() => {
        program.parseAsync(getArgv()).catch(err => {
            if (err instanceof Error) {
                console.log(chalk_1.default.red(`Error: ${err.message}`));
            }
            console.log(err);
        });
    });
}
