"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAddDelegatorParams = exports.getUnsignedAddDelegator = exports.addDelegator = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("@flarenetwork/flarejs/dist/utils");
const utils_2 = require("../utils");
const contracts_1 = require("../constants/contracts");
function checkMaximumAllowedDelegation(ctx, nodeId) {
    return __awaiter(this, void 0, void 0, function* () {
        const delegationDetails = yield (0, utils_2.delegationAddressCount)(ctx);
        if (delegationDetails.validatorNodeId.includes(nodeId.toLowerCase())) {
            return;
        }
        if (delegationDetails.count >= contracts_1.maxAllowedDelegation) {
            throw new Error(`Exceeded maximum allowed delegation of ${contracts_1.maxAllowedDelegation}`);
        }
    });
}
/**
 * @description - Delegate funds to a given validator
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 */
function addDelegator(ctx, nodeID, stakeAmount, startTime, endTime, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        yield checkMaximumAllowedDelegation(ctx, nodeID);
        const params = yield getAddDelegatorParams(ctx, nodeID, stakeAmount, startTime, endTime, threshold);
        const unsignedTx = yield ctx.pchain.buildAddDelegatorTx(...params);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.addDelegator = addDelegator;
/**
 * @description - Generate an unisgned transaction for the add-delegator transaction
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 */
function getUnsignedAddDelegator(ctx, nodeID, stakeAmount, startTime, endTime, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        yield checkMaximumAllowedDelegation(ctx, nodeID);
        const params = yield getAddDelegatorParams(ctx, nodeID, stakeAmount, startTime, endTime, threshold);
        const unsignedTx = yield ctx.pchain.buildAddDelegatorTx(...params);
        return {
            transactionType: 'delegate',
            serialization: (0, utils_2.serializeUnsignedTx)(unsignedTx),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex')
        };
    });
}
exports.getUnsignedAddDelegator = getUnsignedAddDelegator;
/**
 * @description - Generate the parameters for getUnsignedAddDelegator and addDelegator
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 */
function getAddDelegatorParams(ctx, nodeID, stakeAmount, startTime, endTime, threshold = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const locktime = new dist_1.BN(0);
        const asOf = (0, utils_1.UnixNow)();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs(ctx.pAddressBech32);
        const utxoSet = platformVMUTXOResponse.utxos;
        return [
            utxoSet,
            [ctx.pAddressBech32],
            [ctx.pAddressBech32],
            [ctx.pAddressBech32],
            nodeID,
            startTime,
            endTime,
            stakeAmount,
            [ctx.pAddressBech32],
            locktime,
            threshold,
            undefined,
            asOf
        ];
    });
}
exports.getAddDelegatorParams = getAddDelegatorParams;
