"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAddValidatorParams = exports.getUnsignedAddValidator = exports.addValidator = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("@flarenetwork/flarejs/dist/utils");
const utils_2 = require("../utils");
/**
 * @description - Stake by registring your node for validation
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 * @param delegationFee - the fee you charge for delegating to your node
 */
function addValidator(ctx, nodeID, stakeAmount, startTime, endTime, delegationFee, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getAddValidatorParams(ctx, nodeID, stakeAmount, startTime, endTime, delegationFee, threshold);
        const unsignedTx = yield ctx.pchain.buildAddValidatorTx(...params);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.addValidator = addValidator;
/**
 * @description - Get hashes that need to get signed for the addValidator transaction
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 * @param delegationFee - the fee you charge for delegating to your node
 */
function getUnsignedAddValidator(ctx, nodeID, stakeAmount, startTime, endTime, delegationFee, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getAddValidatorParams(ctx, nodeID, stakeAmount, startTime, endTime, delegationFee, threshold);
        const unsignedTx = yield ctx.pchain.buildAddValidatorTx(...params);
        return {
            transactionType: 'stake',
            serialization: (0, utils_2.serializeUnsignedTx)(unsignedTx),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex')
        };
    });
}
exports.getUnsignedAddValidator = getUnsignedAddValidator;
/**
 * @description - Get the parameters for getUnsignedAddValidator and addValidator
 * @param ctx - context with constants initialized from user keys
 * @param nodeID - id of the node you are running (can get it via rpc call)
 * @param stakeAmount - the amount of funds to stake during the node's validation
 * @param startTime - start time of the node's validation
 * @param endTime - end time of the node's validation
 * @param delegationFee - the fee you charge for delegating to your node
 */
function getAddValidatorParams(ctx, nodeID, stakeAmount, startTime, endTime, delegationFee, threshold = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const locktime = new dist_1.BN(0);
        const asOf = (0, utils_1.UnixNow)();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs(ctx.pAddressBech32);
        const utxoSet = platformVMUTXOResponse.utxos;
        return [
            utxoSet,
            [ctx.pAddressBech32],
            [ctx.pAddressBech32],
            [ctx.pAddressBech32],
            nodeID,
            startTime,
            endTime,
            stakeAmount,
            [ctx.pAddressBech32],
            delegationFee,
            locktime,
            threshold,
            undefined,
            asOf
        ];
    });
}
exports.getAddValidatorParams = getAddValidatorParams;
