"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImportPCParams = exports.getExportCPParams = exports.issueSignedEvmTx = exports.issueSignedEvmTxPCImport = exports.issueSignedEvmTxCPExport = exports.getUnsignedImportTxPC = exports.getUnsignedExportTxCP = exports.importTxPC = exports.exportTxCP = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("@flarenetwork/flarejs/dist/utils");
const utils_2 = require("../utils");
/**
 * @description  Exports funds from C-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export from C-chain to P-chain
 * @param fee - export transaction fee
 * @param nonce - export transaction nonce
 */
function exportTxCP(ctx, amount, fee, nonce, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getExportCPParams(ctx, amount, fee, nonce, threshold);
        const unsignedTx = yield ctx.cchain.buildExportTx(...params);
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        const usedFee = params[9].toString();
        return { txid: txid, usedFee: usedFee };
    });
}
exports.exportTxCP = exportTxCP;
/**
 * @description  Import funds exported from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param fee - import transaction fee
 */
function importTxPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getImportPCParams(ctx, fee);
        const unsignedTx = yield ctx.cchain.buildImportTx(...params);
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        const usedFee = params[5].toString();
        return { txid: txid, usedFee: usedFee };
    });
}
exports.importTxPC = importTxPC;
/**
 * Get hashes that need to get signed in order for funds to be
 * exported from P-chain to C-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export from C-chain to P-chain
 * @param fee - export transaction fee
 * @param nonce - export transaction nonce
 */
function getUnsignedExportTxCP(ctx, amount, fee, nonce, threshold) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getExportCPParams(ctx, amount, fee, nonce, threshold);
        const unsignedTx = yield ctx.cchain.buildExportTx(...params);
        return {
            transactionType: 'exportCP',
            serialization: (0, utils_2.serializeExportCP_args)(params),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex'),
            usedFee: params[9].toString()
        };
    });
}
exports.getUnsignedExportTxCP = getUnsignedExportTxCP;
/**
 * @description  Generate unsigned import transaction from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param fee - import transaction fee
 */
function getUnsignedImportTxPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const params = yield getImportPCParams(ctx, fee);
        const unsignedTx = yield ctx.cchain.buildImportTx(...params);
        return {
            transactionType: 'importPC',
            serialization: (0, utils_2.serializeImportPC_args)(params),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex'),
            usedFee: params[5].toString()
        };
    });
}
exports.getUnsignedImportTxPC = getUnsignedImportTxPC;
/**
 * @description Issue a transaction to export funds from C-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 * @param signedTxJson - signed transaction
 */
function issueSignedEvmTxCPExport(ctx, signedTxJson) {
    return __awaiter(this, void 0, void 0, function* () {
        return issueSignedEvmTx(ctx, signedTxJson, (serialization) => __awaiter(this, void 0, void 0, function* () { return ctx.cchain.buildExportTx(...(0, utils_2.deserializeExportCP_args)(serialization)); }));
    });
}
exports.issueSignedEvmTxCPExport = issueSignedEvmTxCPExport;
/**
 * @description Issue a transaction to import funds from P-chain
 * @param ctx - context with constants initialized from user keys
 * @param signedTxJson - signed transaction
 */
function issueSignedEvmTxPCImport(ctx, signedTxJson) {
    return __awaiter(this, void 0, void 0, function* () {
        return issueSignedEvmTx(ctx, signedTxJson, (serialization) => __awaiter(this, void 0, void 0, function* () { return ctx.cchain.buildImportTx(...(0, utils_2.deserializeImportPC_args)(serialization)); }));
    });
}
exports.issueSignedEvmTxPCImport = issueSignedEvmTxPCImport;
/**
 * @description - issues signed transaction
 * @param ctx - context file
 * @param signedTxJson - signed json object
 * @param txBuilder
 * @returns - issues trx and returns the transactionid
 */
function issueSignedEvmTx(ctx, signedTxJson, txBuilder) {
    return __awaiter(this, void 0, void 0, function* () {
        const signatures = Array(signedTxJson.signatureRequests.length).fill(signedTxJson.signature);
        const ecdsaSignatures = signatures.map((signature) => (0, utils_2.expandSignature)(signature));
        const unsignedTx = yield txBuilder(signedTxJson.serialization);
        const tx = unsignedTx.signWithRawSignatures(ecdsaSignatures, ctx.cKeychain);
        const chainTxId = yield ctx.cchain.issueTx(tx);
        return { chainTxId: chainTxId };
    });
}
exports.issueSignedEvmTx = issueSignedEvmTx;
/**
 * @description - generates params to export funds from C to P
 * @param ctx - context file
 * @param amount - amount to be exported
 * @param fee - fees
 * @param nonce - nonce if any
 * @param threshold
 * @returns returns the params need to export fund from C to P chain
 */
function getExportCPParams(ctx, amount, fee, nonce, threshold = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const txcount = yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex);
        const locktime = new dist_1.BN(0);
        const importFee = ctx.pchain.getDefaultTxFee();
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        if (amount == undefined) {
            // fetch cchain balance
            let balance = ((0, utils_2.toBN)(yield ctx.web3.eth.getBalance(ctx.cAddressHex))).toString();
            // if balance is zero, throw error
            if (parseInt(balance) == 0) {
                throw new Error("Account balance is zero");
            }
            // if balance is there, convert it to integer. Since toBN throws error for number with decimal, take the first digits
            balance = (0, utils_2.integerToDecimal)(balance, 9).split(".")[0];
            // convert it o big number
            balance = (0, utils_2.toBN)(balance);
            // buffer amount for gas fees
            const bufferAmount = (0, utils_2.toBN)(11230);
            // based on whether fees
            const exportFees = fee ? bufferAmount === null || bufferAmount === void 0 ? void 0 : bufferAmount.mul(fee) : bufferAmount === null || bufferAmount === void 0 ? void 0 : bufferAmount.mul(baseFee);
            // add it to total expenditure
            const totalExpenditure = importFee.add(exportFees);
            // since it will need to execute substract it from the balance
            amount = balance.sub(totalExpenditure);
        }
        const params = [
            amount.add(importFee),
            ctx.avaxAssetID,
            ctx.pChainBlockchainID,
            ctx.cAddressHex,
            ctx.cAddressBech32,
            [ctx.pAddressBech32],
            nonce !== null && nonce !== void 0 ? nonce : txcount,
            locktime,
            threshold,
            fee !== null && fee !== void 0 ? fee : baseFee
        ];
        const unsignedTx = yield ctx.cchain.buildExportTx(...params);
        const exportCost = (0, utils_1.costExportTx)(unsignedTx);
        // if fee not passed -> use the default
        if (!fee) {
            params[9] = baseFee.mul(new dist_1.BN(exportCost));
        }
        // else use the custom fees passed by the user
        else {
            params[9] = fee;
        }
        return params;
    });
}
exports.getExportCPParams = getExportCPParams;
/**
 * @description - generates params to imports funds from P to C
 * @param ctx - context file
 * @param fee - fee is any
 * @returns - returns the params that will be needed to import funds from P to C chain
 */
function getImportPCParams(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        const evmUTXOResponse = yield ctx.cchain.getUTXOs([ctx.cAddressBech32], ctx.pChainBlockchainID);
        const utxoSet = evmUTXOResponse.utxos;
        const params = [
            utxoSet,
            ctx.cAddressHex,
            [ctx.cAddressBech32],
            ctx.pChainBlockchainID,
            [ctx.cAddressBech32],
            baseFee
        ];
        const unsignedTx = yield ctx.cchain.buildImportTx(...params);
        const importCost = (0, utils_1.costImportTx)(unsignedTx);
        if (!fee) {
            params[5] = baseFee.mul(new dist_1.BN(importCost));
        }
        else {
            params[5] = fee;
        }
        return params;
    });
}
exports.getImportPCParams = getImportPCParams;
